#define MODULE "time"

#include "utils/Time.h"
#include "log/Log.h"

int isleap(int year)
{
    return year % 4 == 0 && (year % 100 != 0 || year % 400 == 0);
}

int make_yday(int year, int month)
{
    static const short int upto[12] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
    int yd;

    yd = upto[month - 1];
    if (month > 2 && isleap(year)) yd++;
    return yd;
}

uint64_t broken_down_to_unix(uint64_t year, uint64_t yday, uint64_t hour, uint64_t min, uint64_t sec)
{
    return sec + min * 60 + hour * 3600 + yday * 86400 + (year - 70) * 31536000 + ((year - 69) / 4) * 86400 -
           ((year - 1) / 100) * 86400 + ((year + 299) / 400) * 86400;
}

static int bcd_number_to_decimal(int num)
{
    return ((num >> 4) * 10) + (num & 0xf);
}

uint64_t unix_boottime(uint8_t boottime[8])
{
    int year = bcd_number_to_decimal(boottime[0]) * 100 + bcd_number_to_decimal(boottime[1]);
    int month = bcd_number_to_decimal(boottime[2]);
    int day = bcd_number_to_decimal(boottime[3]);
    int hour = bcd_number_to_decimal(boottime[4]);
    int minute = bcd_number_to_decimal(boottime[5]);
    int second = bcd_number_to_decimal(boottime[6]);
    kinfoln("UTC boot time: %d-%d-%d %d:%d:%d", year, month, day, hour, minute, second);
    return broken_down_to_unix(year - 1900, make_yday(year, month) + (day - 1), hour, minute, second);
}